#!/usr/bin/env python3
from functools import partial, wraps
import subprocess as sp
import argparse
import logging
import shlex
import sys


def klaxon(message: str= '', title="Klaxon", subtitle="", sound=""):
    """
    Wraps osascript.

    see https://apple.stackexchange.com/questions/57412/how-can-i-trigger-a-notification-center-notification-from-an-applescript-or-shel/115373#115373
    """
    if not sys.platform == 'darwin':
        logging.error('klaxon only works on Mac OS')
        return

    command = f"""osascript -e 'display notification "{message}" with title "{title}" subtitle "{subtitle}" sound name "{sound}"'"""
    sp.run(shlex.split(command))


def klaxonify(
    func=None,
    title="Klaxon",
    message="",
    subtitle=None,
    sound="",
    output_as_message=False,
):
    """
    Send a notification at the termination of a function.

    Args:
        func: the function to be decorated
        title: the notification title
        message: the notification message body
        subtitle: the notifiction subtitle
        sound: the notification sound
        output_as_message (bool): use the decorated function's output as the message body

    Returns: decorated function

    """

    def decorator(function):
        @wraps(function)
        def inner(*args, **kwargs):
            result = function(*args, **kwargs)
            klaxon(
                subtitle=subtitle if subtitle is not None else function.__name__,
                message=message if not output_as_message else result,
                title=title,
                sound=sound,
            )
            return result

        return inner

    if func is not None:
        return decorator(func)
    else:
        return decorator


def main():
    """Parse arguments from command line and pass to notify function."""
    parser = argparse.ArgumentParser(
        prog="klaxon", description="Send Mac OS notifications through osascript."
    )

    parser.add_argument("--message", default="", help="The body of the notification")
    parser.add_argument("--title", default="Klaxon", help="The notification's title")
    parser.add_argument("--subtitle", default="", help="The notification's subtitle")
    parser.add_argument("--sound", help="The sound the notification makes")

    read_stdin = sys.argv.pop() if sys.argv[-1].strip() == "--" else None

    args = parser.parse_args()

    klaxon_ = partial(
        klaxon, title=args.title, subtitle=args.subtitle, sound=args.sound
    )

    if read_stdin is None:
        klaxon_(args.message)
    else:
        with sys.stdin as fd:
            klaxon_(fd.read())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['klaxon']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['klaxon = klaxon:main']}

setup_kwargs = {
    'name': 'klaxon',
    'version': '0.1.9',
    'description': 'Use osascript to send notifications.',
    'long_description': '# klaxon\n\nSend Mac OS notifications from the terminal or Python programs.\n\nThis is useful for when you want a push notification \nfor some long-running background task.\n\nThis is similar to the [terminal-notifier ruby gem][terminal-notifier],\nbut posix-compliant and with fewer features (PR\'s welcome).\n\n## Installation\nFor command-line use, the recommended method of installation is through [pipx].\n```bash\npipx install klaxon\n```\nNaturally, klaxon can also be pip-installed.\n```bash\npip install klaxon\n```\n\n## Usage\n\n### terminal\n\n```bash\n# blank notification\nklaxon\n# with custom message\nklaxon --message "this is the message body"\n# pipe message from other program\necho "this is the message body" | klaxon --\n```\n\n### python\n\n```python\nfrom klaxon import klaxon, klaxonify\n\n# send a notification\n\nklaxon(\n    title=\'hello, klaxon\',\n    subtitle=\'hola\',\n    message=\'it me\'\n)\n\n# we can decorate our functions to have\n# them send notifications at termination\n\n@klaxonify\ndef hello(name=\'world\'):\n    return f\'hello, {name}\'\n\n@klaxonify(title=\'oh hai\', output_as_message=True)\ndef foo():\n    return "This will be the message body."\n\n```\n\n## Security Notice\n\n\n**DO NOT** send untrusted input through klaxon. \n\nSomeone could use a shell escape sequence to execute arbitrary code\non your machine as klaxon functions by invoking [osascript]\nvia a subprocess call.\n\nYou have been warned.\n\n[terminal-notifier]: https://github.com/julienXX/terminal-notifier\n[pipx]: https://github.com/pipxproject/pipx\n[osascript]: https://apple.stackexchange.com/questions/57412/how-can-i-trigger-a-notification-center-notification-from-an-applescript-or-shel/115373#115373\n',
    'author': 'Stephan Fitzpatrick',
    'author_email': 'knowsuchagency@gmail.com',
    'url': 'https://pypi.org/project/klaxon/',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
